﻿using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.FoodMenu.Dtos;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Printer;
using BPA.KitChen.StoreManagement.Core.Common;
using BPA.KitChen.StoreManagement.Core.Entity;
using BPA.KitChen.StoreManagement.Core.Enum;
using BPA.KitChen.StoreManagement.SqlSugar;
using Furion;
using Furion.DatabaseAccessor;
using Furion.DependencyInjection;
using Furion.DynamicApiController;
using Furion.JsonSerialization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.FoodMenu
{

    [ApiDescriptionSettings("店铺管理", Tag = "菜谱管理", SplitCamelCase = false)]
    public class FoodMenuServices : SupperRepository, IFoodMenuServices, IDynamicApiController, ITransient
    {
        /// <summary>
        /// 核心对象：拥有完整的SqlSugar全部功能
        /// </summary>
        private readonly SqlSugarScope db;

        private readonly IHttpContextAccessor _httpContextAccessor;
        /// <summary>
        /// construct
        /// </summary>
        /// <param name="sqlSugarRepository"></param>
        public FoodMenuServices(IHttpContextAccessor httpContextAccessor)
        {
            db = SqlSugarDb.Db;
            _httpContextAccessor = httpContextAccessor;
        }

        /// <summary>
        /// 获取门店的菜谱
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/GetShopFoodMenuPage")]
        public async Task<PageUtil> GetShopFoodMenuPageAsync(StoreFoodMenuPageInputDto inputDto)
        {
           
            int total = 0;
            var data = db.Queryable<BPA_FoodMenuInfo>()
                .Where(a => a.IsDeleted == 0&&a.StoreId==inputDto.StoreId )
                .OrderBy(a => a.CreateAt, OrderByType.Desc)
                .Select(a => new
                {
                    Id = a.Id,
                    Name = a.Name,
                    Sort=a.Sort,
                })
                .ToPageList(inputDto.Current, inputDto.PageSize, ref total);

            return new PageUtil()
            {
                Data = data,
                Total = total

            };
        }



        /// <summary>
        /// 添加修改菜谱
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/AddUpdateFoodMenu")]
        public async Task<bool> AddUpdateFoodMenuAsync(FoodMenuInfoBaseDto inputDto)
        {
            var obj1 = await db.Queryable<BPA_FoodMenuInfo>()
                .Where(x => x.Code == inputDto.Code || x.Name == inputDto.Name).FirstAsync();
            if (!string.IsNullOrWhiteSpace(inputDto.Id))
            {
                var data = await db.Queryable<BPA_FoodMenuInfo>().Where(x => x.Id == inputDto.Id).FirstAsync();


                if (obj1 != null && data.Id != obj1.Id)
                {
                    throw new Exception("名称或编码重复");
                }

                if (data != null)
                {
                    var obj = await db.Queryable<BPA_FoodMenuInfo>()
                        .Where(x => x.Code == inputDto.Code && x.Id != inputDto.Id).FirstAsync();
                    if (obj != null)
                    {

                        return false;
                    }
                    data.Name = inputDto.Name;
                    data.Code = inputDto.Code;
                    data.Sort = inputDto.Sort;
                    data.Status = (CommonStatus)inputDto.Status;
                    var res = await db.Updateable<BPA_FoodMenuInfo>(data).CallEntityMethod(m => m.Modify()).ExecuteCommandAsync();
                    return res > 0;
                }
                else
                {
                    var obj = await db.Queryable<BPA_FoodMenuInfo>()
                        .Where(x => x.Code == inputDto.Code).FirstAsync();
                    if (obj != null)
                    {
                        return false;
                    }
                    var res = await db.Insertable(new BPA_FoodMenuInfo
                    {
                        StoreId = inputDto.StoreId,
                        Name = inputDto.Name,
                        Code = inputDto.Code,
                        Sort = inputDto.Sort,
                        Id = Guid.NewGuid().ToString(),
                        Status = (CommonStatus)inputDto.Status

                    }).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    return res > 0;
                }
            }
            else
            {

                if (obj1 != null && string.IsNullOrEmpty(inputDto.Id))
                {
                    throw new Exception("名称或编码重复");
                }
                var obj = await db.Queryable<BPA_FoodMenuInfo>()
                    .Where(x => x.Code == inputDto.Code).FirstAsync();
                if (obj != null)
                {
                    return false;
                }

                var res = await db.Insertable(new BPA_FoodMenuInfo
                {
                    StoreId = inputDto.StoreId,
                    Name = inputDto.Name,
                    Code = inputDto.Code,
                    Sort = inputDto.Sort,
                    Id = Guid.NewGuid().ToString(),
                    Status = (CommonStatus)inputDto.Status

                }).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                return res > 0;
            }

        }


        /// <summary>
        /// 获取菜品列表
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/GetGoodsByMenuIdPage")]
        public async Task<PageUtil> GetGoodsByMenuIdPage(FoodMenuQueryInputDto inputDto)
        {
            var headers = _httpContextAccessor.HttpContext.Request.Headers;
            var authorization = headers["Authorization"];

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("Authorization", authorization);
            //var url1 = App.Configuration["SAAS_Manage"] + "/api/goods/getgoodslist";
            //var json1 = HttpHelper.HttpGet(url1, "", dic, "");

            var url = App.Configuration["SAAS_Manage"] + "/api/goods/page";
            var json2 = HttpHelper.PostData(url, JsonConvert.SerializeObject(new
            {
                current= inputDto.Current,
                pageSize= inputDto.PageSize,

            }), Encoding.UTF8, "application/json", dic);

            var data=JsonConvert.DeserializeObject<BaseResultDto>(json2);


            var result = new List<GoodsInfoCheck>();
            int total = 0;

            var foodMenuList = await db.Queryable<BPA_FoodMenu>().Where(c => c.FoodMenuId == inputDto.FoodMenuId).ToListAsync();
            foreach (var item in data?.data?.data)
            {
                var check = foodMenuList.Any(x => x.FoodMenuId == item.id);

                result.Add(new GoodsInfoCheck()
                {
                    Id = item.id,
                    Name = item.name,
                    Check = check
                });
            }
            return new PageUtil()
            {
                Data = result,
                Total = data.data.total,
            };
        }

        /// <summary>
        /// 获取菜品列表
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/GetFoodMenuGoodsPage_V2")]
        public async Task<PageUtil> GetFoodMenuGoodsPageAsync_V2(FoodMenuQueryInputDto inputDto)
        {
            int total = 0;
            var data = db.Queryable<BPA_FoodMenu>()
                .Where((a) => a.IsDeleted == 0  && a.FoodMenuId == inputDto.FoodMenuId)
                .OrderBy((a) => a.CreateAt, OrderByType.Desc)
                .Select((a) => new
                {
                    Id = a.Id,
                    Goodid =a.GoodsId,
                    Name =a.GoodsName
                })
                .ToPageList(inputDto.Current, inputDto.PageSize, ref total);

            return new PageUtil()
            {
                Data = data,
                Total = total

            };
        }

        /// <summary>
        /// 编辑菜谱商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        [HttpPost("/api/FoodMenu/EditFoodMenuGoods")]
        public async Task<bool> EditFoodMenuGoods(FoodMenuBaseDto inputDto)
        {
            try
            {
                var headers = _httpContextAccessor.HttpContext.Request.Headers;
                var authorization = headers["Authorization"];

                Dictionary<string, string> dic = new Dictionary<string, string>();
                dic.Add("Authorization", authorization);
                var url1 = App.Configuration["SAAS_Manage"] + "/api/goods/getgoodslist";
                var json1 = HttpHelper.HttpGet(url1, "", dic, "");

                var data2 = JsonConvert.DeserializeObject<BaseResultDto2>(json1);

                db.Ado.BeginTran();
                //删除设置的菜谱
                await db.Deleteable<BPA_FoodMenu>()
                    .Where(x => inputDto.FoodMenuId.Contains(x.FoodMenuId)).ExecuteCommandAsync();
                //菜谱添加商品
                var data = inputDto.GoodsIds.Select(x => new BPA_FoodMenu()
                {
                    FoodMenuId = inputDto.FoodMenuId,
                    GoodsName= data2?.data?.First(c=>c.id==x.GoodsId)?.name,
                    GoodsId = x.GoodsId,
                }).ToList();
                if (data.Count > 0)
                {
                    var res = await db.Insertable(data).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }
                //查询出店铺关联的当前菜谱菜品

                var storeFoodMenu = await db.Queryable<BPA_StoreFoodMenu>()
                    .Where(x => x.FoodMenuId == inputDto.FoodMenuId).ToListAsync();

                var res1 = db.Deleteable<BPA_StoreFoodMenu>().Where(x => x.FoodMenuId == inputDto.FoodMenuId).ExecuteCommand();

                foreach (var item in storeFoodMenu.GroupBy(x => x.StoreId))
                {
                    var thisData = storeFoodMenu.Where(x => x.StoreId == item.First().StoreId).ToList();
                    foreach (var item2 in inputDto.GoodsIds)
                    {
                        var tihsAdd = thisData.Where(x => x.FoodId == item2.GoodsId).FirstOrDefault();

                        await db.Insertable(new BPA_StoreFoodMenu()
                        {
                            CookingWorkId = tihsAdd?.CookingWorkId ?? "",
                            Discount2Price = tihsAdd?.Discount2Price ?? 0,
                            DiscountPrice = tihsAdd?.DiscountPrice ?? 0,
                            FoodId = tihsAdd?.FoodId ?? item2.GoodsId,
                            FoodMenuId = inputDto.FoodMenuId,
                            FoodName= item?.First()?.FoodName,
                            Price = tihsAdd?.Price ?? 0,
                            StopSales = tihsAdd?.StopSales ?? 0,
                            StoreId = item.First().StoreId,
                            Vip2Price = tihsAdd?.Vip2Price ?? 0,
                            VipPrice = tihsAdd?.VipPrice ?? 0,
                        }).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();

                    }

                }

                await StoreAddFoodMenuGoodsNewAsync(new StoreFoodMenuNewDto()
                {
                    StoreId= inputDto.StoreId,
                    FoodMenuIds=new List<string>
                    {
                        inputDto.FoodMenuId
                    }
                });

                db.Ado.CommitTran();
            }
            catch (Exception ex)
            {
                db.Ado.RollbackTran();
                throw new Exception("编辑错误");
            }

            return true;

        }

        [HttpPost("/api/FoodMenu/GetStoreGoods")]
        public async Task<PageUtil> GetStoreGoodsAsync(StoreGoodsQueryInputDto inputDto)
        {
            var total = new RefAsync<int>();
            var data = await db
                .Queryable<BPA_StoreFoodMenu>()
                .Where((s) => s.IsDeleted == 0 )
                .Where((s) => s.StoreId == inputDto.StoreId)
                .WhereIF(!string.IsNullOrEmpty(inputDto.FoodMenuId), (s) => s.FoodMenuId == inputDto.FoodMenuId)
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.GoodsName), (s) => s.FoodName.Contains(inputDto.GoodsName))
                .WhereIF(inputDto.IsDevice != null, (s) => s.IsDevice == inputDto.IsDevice)
                .WhereIF(inputDto.Status != null, (s) => s.Status == inputDto.Status)
                .WhereIF(inputDto.StopSales != null, (s) => s.StopSales == inputDto.StopSales)
                .OrderBy((s) => s.IsDevice, OrderByType.Asc)
                .Select((s) => new StoreGoodsBaseDto()
                {
                    Id = s.Id.SelectAll(),
                    GoodsId = s.FoodId,
                    Goods_Name =s.FoodName
                })
                .ToPageListAsync(inputDto.Current, inputDto.PageSize, total);
            return new PageUtil()
            {
                Data = data,
                Total = total
            };
        }


        /// <summary>
        /// 店铺添加菜谱new
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/FoodMenu/StoreAddFoodMenuNew")]
        public async Task<bool> StoreAddFoodMenuGoodsNewAsync(StoreFoodMenuNewDto inputDto)
        {
            var data = db.Queryable<BPA_FoodMenu>()
                .Where((a) => a.IsDeleted == 0 &&  inputDto.FoodMenuIds.Contains(a.FoodMenuId))
                .OrderBy((a) => a.CreateAt, OrderByType.Desc)
                .Select((a) => new
                {
                    Id = a.Id,
                    goodsId =a.Id,
                    Price = 0,
                    vipPrice =0,
                    a.FoodMenuId
                }).ToList();



            //删除门店下所有菜谱商品
            db.Deleteable<BPA_StoreFoodMenu>().Where(x => x.StoreId == inputDto.StoreId).ExecuteCommand();

            List<BPA_StoreFoodMenu> bPA_StoreFoods = new List<BPA_StoreFoodMenu>();

            foreach (var item in data)
            {
                BPA_StoreFoodMenu bPA_StoreFood = new BPA_StoreFoodMenu();
                bPA_StoreFood.FoodMenuId = item.FoodMenuId;
                bPA_StoreFood.StoreId = inputDto.StoreId;
                bPA_StoreFood.FoodId = item.goodsId;
                bPA_StoreFood.Price = item.Price;
                bPA_StoreFood.VipPrice = item.vipPrice;
                bPA_StoreFood.IsDevice = 0;
                bPA_StoreFoods.Add(bPA_StoreFood);
            }


            if (bPA_StoreFoods.Count > 0)
            {
                return db.Insertable(bPA_StoreFoods).CallEntityMethod(it => it.Create()).ExecuteCommand() > 0;
            }
            else
            {
                return false;
            }
        }

    }
}
