﻿using BPA.KitChen.StoreManagement.Application.Service.ShopManage.Printer.Dtos;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.StoreStaff;
using BPA.KitChen.StoreManagement.Core.Entity;
using BPA.KitChen.StoreManagement.SqlSugar;
using Furion.DependencyInjection;
using Furion.DynamicApiController;
using Mapster;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.Printer
{
    [ApiDescriptionSettings("店铺管理", Tag = "打印机管理", SplitCamelCase = false)]
    public class PrinterServices : SupperRepository, IPrinterServices, IDynamicApiController, ITransient
    {

        /// <summary>
        /// 核心对象：拥有完整的SqlSugar全部功能
        /// </summary>
        private readonly SqlSugarScope db;
        /// <summary>
        /// construct
        /// </summary>
        /// <param name="sqlSugarRepository"></param>
        public PrinterServices()
        {
            db = SqlSugarDb.Db;

        }

        /// <summary>
        /// 添加设备
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/Printer/AddPrinter")]
        public bool AddPrinter(PrinterCreateOrUpdateInputDto inputDto)
        {
            try
            {
                db.BeginTran();
                var obj = inputDto.Adapt<BPA_Printer>();

                //添加设备
                db.Insertable(obj)
                     .CallEntityMethod(m => m.Create()).ExecuteCommand();

                //店铺
                db.Insertable(new BPA_PrinterConfigure()
                {
                    PrinterId = obj.Id,
                    ConfigureType = 2,
                    Value = inputDto.StoreId
                }).CallEntityMethod(m => m.Create()).ExecuteCommand();

                //模板
                db.Insertable(new BPA_PrinterConfigure()
                {
                    PrinterId = obj.Id,
                    ConfigureType = 1,
                    Value = inputDto.TemplaterId
                }).CallEntityMethod(m => m.Create()).ExecuteCommand();

                if (inputDto.GoodsIdList != null)
                {
                    foreach (var item in inputDto.GoodsIdList)
                    {

                        //模板
                        db.Insertable(new BPA_PrinterConfigure()
                        {
                            PrinterId = obj.Id,
                            ConfigureType = 3,
                            Value = item
                        }).CallEntityMethod(m => m.Create()).ExecuteCommand();
                    }
                }

                db.CommitTran();

                return true;
            }
            catch (Exception e)
            {
                db.Ado.RollbackTran();
                return false;
            }
        }
        /// <summary>
        /// 修改设备
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/Printer/UpdatePrinter")]
        public bool UpdatePrinter(PrinterCreateOrUpdateInputDto inputDto)
        {
            try
            {
                db.BeginTran();

                var res = db.Updateable(inputDto.Adapt<BPA_Printer>())
                .CallEntityMethod(m => m.Modify()).ExecuteCommand();
                db.CommitTran();


                var list2 = db.Queryable<BPA_PrinterConfigure>()
                    .Where(x => x.PrinterId == inputDto.Id && x.IsDeleted == 0 && x.ConfigureType == 1).ToList();
                list2.ForEach((item) => { item.IsDeleted = 1; });
                db.Updateable(list2).CallEntityMethod(m => m.Delete()).ExecuteCommand();

                //模板
                db.Insertable(new BPA_PrinterConfigure()
                {
                    PrinterId = inputDto.Id,
                    ConfigureType = 1,
                    Value = inputDto.TemplaterId
                }).CallEntityMethod(m => m.Create()).ExecuteCommand();

                //菜品
                db.Updateable<BPA_PrinterConfigure>()
                    .SetColumns(it => new BPA_PrinterConfigure() { IsDeleted = 1 })
                    .Where(x => x.PrinterId == inputDto.Id && x.ConfigureType == 3)
                    .ExecuteCommand();

                if (inputDto.GoodsIdList != null)
                {
                    foreach (var item in inputDto.GoodsIdList.Distinct())
                    {

                        //模板
                        db.Insertable(new BPA_PrinterConfigure()
                        {
                            PrinterId = inputDto.Id,
                            ConfigureType = 3,
                            Value = item
                        }).CallEntityMethod(m => m.Create()).ExecuteCommand();
                    }
                }

                return true;
            }
            catch (Exception e)
            {
                db.Ado.RollbackTran();
                return false;
            }
        }

        /// <summary>
        ///查询设备
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/Printer/GetPrinterList")]
        public List<PrinterOutDto> GetPrinterList(string storeId)
        {
            //查询当前店铺的打印机和没有绑定的打印机
            var list = db.Queryable<BPA_Printer, BPA_PrinterConfigure, BPA_PrinterConfigure,
                    BPA_PrinterConfigure>((a, b, c, d) =>
                    new JoinQueryInfos(
                        JoinType.Inner, a.Id == b.PrinterId && b.ConfigureType == 2 && b.Value == storeId && b.IsDeleted == 0,
                        JoinType.Left, a.Id == c.PrinterId && c.ConfigureType == 4 && c.IsDeleted == 0,
                        JoinType.Left, a.Id == d.PrinterId && d.ConfigureType == 1 && d.IsDeleted == 0))
                .Where((a, b, c) => a.IsDeleted == 0)
                .Select((a, b, c, d) => new PrinterOutDto
                {
                    Id = a.Id,
                    ApplicationId = a.ApplicationId,
                    Applicationkey = a.Applicationkey,
                    IsHost = c.Value,
                    Name = a.Name,
                    TemplaterId = d.Value,
                    TerminalId = a.TerminalId,
                    TerminalKey = a.TerminalKey,
                    StoreId=b.Value
                }).ToList();

            var goodsId = db.Queryable<BPA_PrinterConfigure>().Where(x =>
                    x.IsDeleted == 0 && x.ConfigureType == 3 && list.Select(y => y.Id).Contains(x.PrinterId))
                .ToList();

            foreach (var item in list)
            {
                var data = goodsId.Where(x => x.PrinterId == item.Id)
                    .Select(x => x.Value).ToList();
                item.goodsIdList = data;

            }

            return list;
        }

        /// <summary>
        /// 删除打印机
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        [HttpPost("/api/Printer/DelPrinter")]
        public bool DelPrinter(List<string> ids)
        {
            try
            {
                db.BeginTran();

                //删除
                var resEntitites = db.Queryable<BPA_Printer>().Where(x => ids.Contains(x.Id)).ToList();
                resEntitites.ForEach((item) => { item.IsDeleted = 1; });
                db.Updateable(resEntitites).CallEntityMethod(m => m.Delete()).ExecuteCommand();

                var list2 = db.Queryable<BPA_PrinterConfigure>().Where(x => ids.Contains(x.PrinterId)).ToList();
                list2.ForEach((item) => { item.IsDeleted = 1; });
                db.Updateable(list2).CallEntityMethod(m => m.Delete()).ExecuteCommand();


                db.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                db.RollbackTran();
                return false;
            }

        }

        /// <summary>
        /// 设为主打印机
        /// </summary>
        /// <param name="storeId"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpPost("/api/Printer/SetPrinterHost")]
        public bool SetPrinterHost(string storeId, string id)
        {

            try
            {
                db.BeginTran();

                var printerConfigure = db.Queryable<BPA_PrinterConfigure>()
                    .Where(x => x.Value == storeId && x.ConfigureType == 2 && x.IsDeleted == 0)
                    .ToList();

                printerConfigure = db.Queryable<BPA_PrinterConfigure>()
                   .Where(x => x.ConfigureType == 4
                   && x.IsDeleted == 0 && printerConfigure.Select(y => y.PrinterId).Contains(x.PrinterId))
                   .ToList();

                printerConfigure.ForEach((item) => { item.IsDeleted = 1; });
                var vv = db.Updateable(printerConfigure).CallEntityMethod(m => m.Delete()).ExecuteCommand();

                var vvvv = db.Insertable(new BPA_PrinterConfigure()
                {

                    ConfigureType = 4,
                    Value = "true",
                    PrinterId = id

                }).CallEntityMethod(m => m.Create())
                    .ExecuteCommand();
                db.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                db.RollbackTran();
                return false;
            }



        }

        /// <summary>
        ///查询店铺菜谱
        /// </summary>
        /// <param name="storeId"></param>
        /// <returns></returns>
        [HttpGet("/api/Printer/GetStoreFoodMenu")]
        public List<StoreFoodMenuOutDtocs> GetStoreFoodMenu(string storeId)
        {
            var list = db.Queryable<BPA_StoreFoodMenu, BPA_FoodMenuInfo>((a, b) =>
                    new JoinQueryInfos(JoinType.Left, a.FoodMenuId == b.Id))
                .Where((a, b) => a.IsDeleted == 0 && a.StoreId == storeId)
                .Select((a, b) => new StoreFoodMenuOutDtocs
                {
                    FoodId = a.FoodId,
                    FoodName =a.FoodName

                }).ToList();
            return list;
        }


        #region 打印机模板
        /// <summary>
        /// 添加打印机模板
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/Printer/AddPrinterTemplate")]
        public bool AddPrinterTemplate(PrinterTemplateCreateOrUpdateInputDto inputDto)
        {
            var re = db.Insertable(inputDto.Adapt<BPA_PrinterTemplate>())
                  .CallEntityMethod(m => m.Create()).ExecuteCommand();

            return re > 0;

        }

        /// <summary>
        /// 修改模板
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPut("/api/Printer/UpdatePrinterTemplate")]
        public bool UpdatePrinterTemplate(PrinterTemplateCreateOrUpdateInputDto inputDto)
        {
            var res = db.Updateable(inputDto.Adapt<BPA_PrinterTemplate>())
                .CallEntityMethod(m => m.Modify()).ExecuteCommand();
            return res > 0;
        }


        /// <summary>
        /// 删除打印机
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        [HttpDelete("/api/Printer/DelPrinterTemplater")]
        public bool DelPrinterTemplater(List<string> ids)
        {
            //删除
            var resEntitites = db.Queryable<BPA_PrinterTemplate>().Where(x => ids.Contains(x.Id)).ToList();
            resEntitites.ForEach((item) => { item.IsDeleted = 1; });
            var res = db.Updateable(resEntitites).CallEntityMethod(m => m.Delete()).ExecuteCommand();
            return res > 0;
        }


        /// <summary>
        /// 获取模板
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/Printer/GetPrinterTemplater")]
        public List<PrinterTemplateOutDto> GetPrinterTemplater()
        {
            var list = db.Queryable<BPA_PrinterTemplate>().Where(x => x.IsDeleted == 0).ToList();
            return list.Adapt<List<PrinterTemplateOutDto>>();
        }

        ///// <summary>
        /////查询店铺菜谱
        ///// </summary>
        ///// <param name="storeId"></param>
        ///// <returns></returns>
        //public List<StoreFoodMenuOutDtocs> GetStoreFoodMenu(string storeId)
        //{
        //    var list = db.Queryable<BPA_StoreFoodMenu, BPA_FoodMenuInfo, BPA_GoodsInfo>((a, b, c) =>
        //          new JoinQueryInfos(JoinType.Left, a.FoodMenuId == b.Id,
        //              JoinType.Left, a.FoodId == c.Id))
        //        .Where((a, b) => a.IsDeleted == 0 && a.StoreId == storeId)
        //        .Select((a, b, c) => new StoreFoodMenuOutDtocs
        //        {
        //            FoodId = a.FoodId,
        //            FoodName = c.Goods_Name

        //        }).ToList();
        //    return list;
        //}
        #endregion



    }
}
