﻿using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.StoreAdvertisement.Dtos;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.StoreDesktopNumber;
using BPA.KitChen.StoreManagement.Core.Common;
using BPA.KitChen.StoreManagement.Core.Common.Const;
using BPA.KitChen.StoreManagement.Core.Entity;
using BPA.KitChen.StoreManagement.SqlSugar;
using Furion;
using Furion.DependencyInjection;
using Furion.DynamicApiController;
using Mapster;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.StoreAdvertisement
{

    [ApiDescriptionSettings("店铺管理", Tag = "店铺广告", SplitCamelCase = false)]
    public class StoreAdvertisementService : SupperRepository, IStoreAdvertisementService, IDynamicApiController, ITransient
    {
        private readonly SqlSugarScope db;
        private readonly IWebHostEnvironment _webHostEnvironment;

        private readonly string host = "";//App.Configuration["HostPort"];
        public StoreAdvertisementService(IWebHostEnvironment webHostEnvironment)
        {
            db = SqlSugarDb.Db;
            _webHostEnvironment= webHostEnvironment;
        }

        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpPost("/api/StoreAdvertisement/AddStoreAdvertisement")]
        public bool AddStoreAdvertisementAsync(StoreAdvertisementEntityDto inputDto)
        {
            string imageUrls = string.Empty;
            string videoUrls = string.Empty;
            //广告类型 0图片 1 视频
            if (inputDto.Type == 0)
            {
                //图片
                if (inputDto.UploadType == 1) //校验：0自定义上传，1网络路径
                {
                    imageUrls = inputDto.NetWorkUrl;
                }
                else
                {
                    if (inputDto != null && inputDto.ImageUrl != null && inputDto.ImageUrl.Count > 0)
                    {
                        foreach (var item in inputDto.ImageUrl)
                        {
                            imageUrls += item.Name + "|";
                        }
                    }
                }
            }
            else
            {
                //视频
                if (inputDto.UploadType == 1) //校验：0自定义上传，1网络路径
                    videoUrls = inputDto.NetWorkUrl;
                else
                    videoUrls = inputDto.VideoUrl;
            }

            BPA_StoreAdvertisement info = new BPA_StoreAdvertisement()
            {
                ImageUrl = imageUrls.TrimEnd('|'),
                Content = inputDto.Content??"",
                VideoUrl = videoUrls,
                UploadType = inputDto.UploadType,
                StartTime = inputDto.StartTime,
                EndTime = inputDto.EndTime,
                Type = inputDto.Type,
                Step = inputDto.Step,
                Title = inputDto.Title,
                StoreId = inputDto.StoreId,
            };
            var res = db.Insertable<BPA_StoreAdvertisement>(info).CallEntityMethod(m => m.Create()).ExecuteCommand();
            return res > 0;
        }

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="inputList"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpDelete("/api/StoreAdvertisement/DelStoreAdvertisement")]
        public async Task<bool> DelStoreAdvertisementAsync(List<string> inputList)
        {
            var deleteBy = App.User.FindFirst(ClaimConst.CLAINM_ACCOUNT)?.Value;
            var res = await db.Updateable<BPA_StoreAdvertisement>(x => new BPA_StoreAdvertisement()
            {
                IsDeleted = 1,
                DeleteAt = DateTime.Now,
                DeleteBy = deleteBy
            })
                .Where(x => inputList.Contains(x.Id))
                .ExecuteCommandAsync();
            return res > 0;
        }

        /// <summary>
        /// 列表，查询，分页
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpPost("/api/StoreAdvertisement/GetStoreAdvertisementPage")]
        public async Task<PageUtil> GetStoreAdvertisementPageAsync(QueryStoreAdvertisementDto inputDto)
        {
            var total = new RefAsync<int>();
            var data = await db.Queryable<BPA_StoreAdvertisement>().Where((x) => x.IsDeleted == 0)
                .WhereIF(!string.IsNullOrWhiteSpace(inputDto.StoreId), x => x.StoreId.Contains(inputDto.StoreId))
                .OrderBy(x => x.CreateAt, OrderByType.Desc)
                .ToPageListAsync(inputDto.Current, inputDto.PageSize, total);

            List<StoreAdvertisementEntityDto> dtos = new List<StoreAdvertisementEntityDto>();
            if (data != null)
            {
                foreach (var item in data)
                {
                    //前端上传控件预览缩略地址
                    List<ImageUrl> imageUrls = new();
                    if (item.Type == 0)
                    {
                        foreach (var item2 in item.ImageUrl.Split('|').ToArray())
                        {
                            ImageUrl imageUrl = new ImageUrl()
                            {
                                Name = item2,
                                Uid = item2,
                                Url = host + item2
                            };
                            imageUrls.Add(imageUrl);
                        }
                    }
                    else if (item.Type == 1)
                    {
                        ImageUrl videoUrl = new ImageUrl()
                        {
                            Name = item.VideoUrl,
                            Uid = item.VideoUrl,
                            Url = host + item.VideoUrl,
                        };
                        imageUrls.Add(videoUrl);
                    }
                    StoreAdvertisementEntityDto dto = new()
                    {
                        Id = item.Id,
                        Title = item.Title,
                        Type = item.Type,
                        Content = item.Content,
                        StartTime = item.StartTime,
                        EndTime = item.EndTime,
                        VideoUrl = item.VideoUrl,
                        StoreId = item.StoreId,
                        Status = ((int)item.Status).ToString(),
                        Step = item.Step,
                        CreateAt = item.CreateAt,
                        ImageUrl = imageUrls,
                        UploadType = item.UploadType,
                        NetWorkUrl = item.UploadType == 1 ? (item.Type == 0 ? item.ImageUrl : item.VideoUrl) : "",
                    };
                    dtos.Add(dto);
                }
            }
            return new PageUtil()
            {
                Data = dtos,
                Total = total
            };
        }

        /// <summary>
        /// 修改
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpPut("/api/StoreAdvertisement/UpdateStoreAdvertisement")]
        public bool UpdateStoreAdvertisementAsync(StoreAdvertisementEntityDto inputDto)
        {
            string imageUrls = string.Empty;
            string videoUrls = string.Empty;
            //广告类型 0图片 1 视频
            if (inputDto.Type == 0)
            {
                //图片
                if (inputDto.UploadType == 1) //校验：0自定义上传，1网络路径
                {
                    imageUrls = inputDto.NetWorkUrl;
                }
                else
                {
                    if (inputDto != null && inputDto.ImageUrl != null && inputDto.ImageUrl.Count > 0)
                    {
                        foreach (var item in inputDto.ImageUrl)
                        {
                            imageUrls += item.Name??"" + "|";
                        }
                    }
                }
            }
            else
            {
                //视频
                if (inputDto.UploadType == 1) //校验：0自定义上传，1网络路径
                    videoUrls = inputDto.NetWorkUrl;
                else
                    videoUrls = inputDto.VideoUrl;
            }
            var data = db.Queryable<BPA_StoreAdvertisement>().Where(x => x.Id == inputDto.Id).First();
            data.UploadType = inputDto.UploadType;
            data.StartTime = inputDto.StartTime;
            data.EndTime = inputDto.EndTime;
            data.Content = inputDto.Content;
            data.Title = inputDto.Title;
            data.Step = inputDto.Step;
            data.Type = inputDto.Type;
            data.ImageUrl = imageUrls.TrimEnd('|');
            data.VideoUrl = videoUrls;

            var res = db.Updateable<BPA_StoreAdvertisement>(data).CallEntityMethod(i => i.Modify()).ExecuteCommand();
            return res > 0;
        }


        /// <summary>
        /// 停用，启用
        /// </summary>
        /// <param name="inputList"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpPost("/api/StoreAdvertisement/UpdateStoreAdvertisementStatus")]
        public async Task<bool> UpdateStoreAdvertisementStatusAsync(StoreAdvertisementEntityDto inputDto)
        {
            var data = await db.Queryable<BPA_StoreAdvertisement>().Where(x => x.Id == inputDto.Id).FirstAsync();

            if (data != null)
            {
                var res = await db.Updateable<BPA_StoreAdvertisement>(inputDto.Adapt<BPA_StoreAdvertisement>()).CallEntityMethod(i => i.Modify())
                       .UpdateColumns(x => new
                       {
                           x.Status,
                           x.UpdateAt,
                           x.UpdateBy
                       })
                       .Where(x => x.Id == inputDto.Id).ExecuteCommandAsync();
                return res > 0;
            }

            return false;
        }

        /// <summary>
        /// 获取广告地址
        /// </summary>
        /// <param name="storeId">地铺id</param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        [HttpPost("/api/StoreAdvertisement/QueryStoreAdvertisementURL")]
        public BPA_StoreAdvertisement QueryStoreAdvertisementURL(string storeId)
        {
            var res = db.Queryable<BPA_StoreAdvertisement>().Where(i => i.IsDeleted == 0 && i.Status == 0 && i.StoreId == storeId).First();

            return res;
        }

        /// <summary>
        /// 上传图片,通过Form表单提交
        /// </summary>
        /// <returns></returns>
        [Route("/api/Upload/FormImg")]
        [HttpPost, AllowAnonymous, NonUnify]
        public ResultEntity UploadImg(IFormFile file, string storeId)
        {
            if (file == null)
                return new ResultEntity()
                {
                    Data = null,
                    IsSuccess = false,
                    Msg = "文件为空"
                };
            if (string.IsNullOrEmpty(storeId) || storeId.Equals("undefined"))
                return new ResultEntity()
                {
                    Data = null,
                    IsSuccess = false,
                    Msg = "请选择店铺"
                };
            //文件后缀
            var fileExtension = Path.GetExtension(file.FileName);

            //判断后缀是否是图片和视频
            const string fileFilt = ".gif|.jpg|.jpeg|.png| .avi| .rmvb| .rm| .mp4| .flv| .mpg|.m4v";
            if (fileExtension == null)
                return new ResultEntity()
                {
                    Data = null,
                    IsSuccess = false,
                    Msg = "上传的文件没有后缀"
                };
            //判断格式
            if (fileFilt.IndexOf(fileExtension.ToLower(), StringComparison.Ordinal) <= -1)
                return new ResultEntity()
                {
                    Data = null,
                    IsSuccess = false,
                    Msg = "请上传图片或视频格式的文件"
                };
            //判断文件大小    
            long length = file.Length;
            if (length > 1024 * 1024 * 20) //20M
                return new ResultEntity()
                {
                    Data = null,
                    IsSuccess = false,
                    Msg = "上传的文件不能大于20M"
                };
            //判断店铺
            string storeName = storeId;//storeInfoService.GetStoreName(storeId);
            if (string.IsNullOrEmpty(storeName))
                return new ResultEntity()
                {
                    Data = null,
                    IsSuccess = false,
                    Msg = "该店铺不存在"
                };

            //返回的文件地址
            List<string> filenames = new List<string>();
            try
            {
                //图片存储路径
                var imagePath = string.Format("/uploadFiles/" + storeName + "/image/");
                //网页存储路径
                var htmlPath = string.Format("/uploadFiles/" + storeName + "/mv/");
                //获取当前web目录
                var webRootPath = _webHostEnvironment.WebRootPath;

                // Console.WriteLine("htmlUrl:" + webRootPath);

                //判断文件夹是否存在,创建文件夹
                string flag = "";
                var filePath = string.Empty;
                if (" .avi| .rmvb| .rm| .mp4| .flv| .mpg|.m4v".IndexOf(fileExtension.ToLower(), StringComparison.Ordinal) >= 1)
                {
                    flag = "mv";
                    filePath = htmlPath;
                }
                else
                {
                    flag = "pic";
                    filePath = imagePath;
                }
                string filePathName = FileHelper.UploadFile(file, webRootPath + filePath);

                //Console.WriteLine("htmlUrlName:" + webRootPath + filePath);
                //返回文件相对路径
                filenames.Add(filePath + file.FileName);

                return new ResultEntity()
                {
                    Data = filenames,
                    IsSuccess = true,
                    Msg = flag
                };
            }
            catch (Exception ex)
            {
                return new ResultEntity()
                {
                    Data = null,
                    IsSuccess = false,
                    Msg = "上传失败：" + ex.Message
                };
            }

        }
    }
}
