﻿using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.StoreDesktopNumber.Dtos;
using BPA.KitChen.StoreManagement.Core.Common.Const;
using BPA.KitChen.StoreManagement.Core.Entity;
using BPA.KitChen.StoreManagement.SqlSugar;
using Furion;
using Furion.DependencyInjection;
using Furion.DynamicApiController;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.StoreDesktopNumber
{

    [ApiDescriptionSettings("店铺管理", Tag = "店铺桌面码", SplitCamelCase = false)]
    public class StoreDesktopNumber : SupperRepository, IStoreDesktopNumber, IDynamicApiController, ITransient
    {
      
        private readonly SqlSugarScope db; // 核心对象：拥有完整的SqlSugar全部功能

        public StoreDesktopNumber()
        {
            db = SqlSugarDb.Db;
        }

        /// <summary>
        /// 添加桌面编码
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/StoreDesktopNumber/Add")]
        public async Task<bool> Add(DesktopNumberInputCreateOrUpdateDto inputDto)
        {
            try
            {
                db.Ado.BeginTran();
                var storeDesktopNumbes = new List<BPA_StoreDesktopNumber>();
                for (int i = inputDto.StartNumber; i <= inputDto.EndNumber; i++)
                {
                    var id = inputDto.Code??"";
                    if (!inputDto.ExcludeNumber.Contains(i))
                    {

                        foreach (var item in inputDto.StoreIds)
                        {
                            storeDesktopNumbes.Add(new BPA_StoreDesktopNumber()
                            {
                                DesktopNumber = id + i.ToString(),
                                StoreId = item,
                                Code = id
                            });
                        }
                    }
                }

                string[] number = storeDesktopNumbes.Select(x => x.DesktopNumber).ToArray();
                if (number.Length <= 0)
                {
                    return false;
                }
                //处理重复插入的桌台号
                List<BPA_StoreDesktopNumber> bPA_StoreDesktopNumbers = db.Queryable<BPA_StoreDesktopNumber>().Where(it => number.Contains(it.DesktopNumber)).ToList();
                foreach (var item in bPA_StoreDesktopNumbers)
                {
                    storeDesktopNumbes.Remove(storeDesktopNumbes.FirstOrDefault(a => a.DesktopNumber == item.DesktopNumber && item.StoreId == a.StoreId));
                }
                await db.Insertable(storeDesktopNumbes).ExecuteCommandAsync();
                db.Ado.CommitTran();
                return true;
            }
            catch (Exception ex)
            {
                db.Ado.RollbackTran();
                return false;
            }

        }

        /// <summary>
        /// 删除桌面码
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        [HttpDelete("/api/StoreDesktopNumber/Delete")]
        public async Task<bool> Delete(List<string> ids)
        {
            try
            {
                db.Ado.BeginTran();
                //删除店铺绑定桌面码
                var storeDesktopNumbes = db.Queryable<BPA_StoreDesktopNumber>()
                    .Where(x => ids.Contains(x.Id) && x.IsDeleted == 0)
                    .ToList();
                //storeDesktopNumbes.ForEach((item) =>
                //{
                //    item.IsDeleted = 1;
                //    item.DeleteAt = DateTime.Now;
                //    item.DeleteBy = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
                //});
                await db.Deleteable(storeDesktopNumbes).ExecuteCommandAsync();
                //await db.Updateable(storeDesktopNumbes)
                //    .UpdateColumns(it => new { it.IsDeleted, it.DeleteBy, it.DeleteAt })
                //    .ExecuteCommandHasChangeAsync();
                db.Ado.CommitTran();
                return true;
            }
            catch (Exception ex)
            {
                db.Ado.RollbackTran();
                return false;
            }

        }

        /// <summary>
        /// 修改桌面码
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPut("/api/StoreDesktopNumber/UpdateStoreDesktopNumber1")]
        public async Task<bool> UpdateStoreDesktopNumber(DesktopNumberInputCreateOrUpdateDto inputDto)
        {
            try
            {
                //db.Ado.BeginTran();
                ////删除店铺绑定桌面码
                //var storeDesktopNumbes = db.Queryable<BPA_StoreDesktopNumber>()
                //    .Where(x => x.Id == inputDto.Id)
                //    .ToList();
                //storeDesktopNumbes.ForEach((item) =>
                //{
                //    item.IsDeleted = 1;
                //    item.DeleteAt = DateTime.Now;
                //    item.DeleteBy = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
                //});
                //await db.Updateable(storeDesktopNumbes)
                //    .UpdateColumns(it => new { it.IsDeleted, it.DeleteBy, it.DeleteAt })
                //    .ExecuteCommandHasChangeAsync();
                //db.Ado.CommitTran();
                ////重新绑定店铺桌面码
                //storeDesktopNumbes = new List<BPA_StoreDesktopNumber>();

                //foreach (var item in inputDto.StoreIds)
                //{
                //    storeDesktopNumbes.Add(new BPA_StoreDesktopNumber()
                //    {
                //        DesktopNumberId = inputDto.Id,
                //        StoreId = item
                //    });
                //}
                //await db.Insertable(storeDesktopNumbes).ExecuteCommandAsync();
                return true;
            }
            catch (Exception ex)
            {
                //db.Ado.RollbackTran();
                return false;
            }

        }

        /// <summary>
        /// 修改桌面码
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPut("/api/StoreDesktopNumber/UpdateStoreDesktopNumber")]
        public async Task<bool> UpdateStoreDesktopNumber(DesktopNumberInputUpdateDto inputDto)
        {
            try
            {
                //删除店铺绑定桌面码
                var storeDesktopNumbes = db.Queryable<BPA_StoreDesktopNumber>()
                    .Where(x => x.Id == inputDto.Id)
                    .First();
                if (storeDesktopNumbes == null) return false;
                storeDesktopNumbes.DesktopNumber = inputDto.DesktopNumber;
                storeDesktopNumbes.Code = inputDto.Code;
                return await db.Updateable(storeDesktopNumbes).ExecuteCommandAsync() > 0;
            }
            catch (Exception ex)
            {
                return false;
            }

        }

        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/StoreDesktopNumber/Page")]
        public async Task<PageUtil> Page(DesktopNumberPageInputDto inputDto)
        {
            var total = new RefAsync<int>();
            List<IConditionalModel> conModels = new List<IConditionalModel>();
            if (!string.IsNullOrEmpty(inputDto.storeName))//物料名称
            {
                conModels.Add(new ConditionalModel() { FieldName = "b.Name", ConditionalType = ConditionalType.Like, FieldValue = inputDto.storeName });
            }
            string groupId = App.User.FindFirst(ClaimConst.GroupId)?.Value;

         
            var data = await db.Queryable<BPA_StoreDesktopNumber, ShopEntity>((a, b) => new JoinQueryInfos(
                   JoinType.Left, a.StoreId == b.Id))
                .Where((a, b) => a.IsDeleted == 0)
                 .Where(conModels)
                .WhereIF(!string.IsNullOrEmpty(inputDto.storeId),(a,b)=>a.StoreId==inputDto.storeId)
                .OrderBy((a, b) => new { a.DesktopNumber, a.StoreId }, OrderByType.Asc)
                .Select((a, b) => new DesktopNumberOutDto() { Id = a.Id.SelectAll(), StoreName = b.Name, Host = "" })
                .ToPageListAsync(inputDto.Current, inputDto.PageSize, total);

            return new PageUtil()
            {
                Data = data,
                Total = total
            };

        }
    }
}
