﻿using Furion;
using Furion.DependencyInjection;
using Furion.FriendlyException;
using Mapster;
using SqlSugar;
using Furion.DataEncryption;
using BPA.KitChen.StoreManagement.SqlSugar;
using Furion.DynamicApiController;
using BPA.KitChen.StoreManagement.Application.Service.ShopManage.StoreStaff.Dtos;
using BPA.KitChen.StoreManagement.Core.Common.Const;
using BPA.KitChen.StoreManagement.Core.Entity;
using BPA.KitChen.StoreManagement.Application.BaseDto;
using BPA.KitChen.StoreManagement.Core.Enum;
using Microsoft.AspNetCore.Mvc;
using Furion.DatabaseAccessor;

namespace BPA.KitChen.StoreManagement.Application.Service.ShopManage.StoreStaff
{

    [ApiDescriptionSettings("店铺管理", Tag = "员工管理", SplitCamelCase = false)]
    public class StoreStaffService : SupperRepository, IStoreStaffService, IDynamicApiController, ITransient
    {
        private readonly SqlSugarScope db;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sqlSugarRepository"></param>
        public StoreStaffService()
        {
            db = SqlSugarDb.Db;   // 推荐操作


        }

        [HttpPost("/api/StoreStaff/AddStoreStaff")]
        public async Task<bool> AddStoreStaff(CreateOrUpdateStoreStaffInputDto inputDto)
        {
          
            var data=await db.Queryable<BPA_StoreStaff>()
                .Where(x => x.Number == inputDto.Number&&x.IsDeleted==0)
                .ToListAsync();
            if (data.Count>0)
            {
                throw Oops.Oh($"{inputDto.Number} 职工号重复");
            }

            var inputdata = inputDto.Adapt<BPA_StoreStaff>();
            inputdata.Pwd= MD5Encryption.Encrypt("123456");
            var res = await db.Insertable(inputdata).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
            return res > 0;

        }

        [HttpDelete("/api/StoreStaff/DelStoreStaff")]
        public async Task<bool> DelStoreStaff(List<string> ids)
        {
            var data = await db.Queryable<BPA_StoreStaff>()
                .Where((x) => x.IsDeleted == 0)
                .Where(x => ids.Contains(x.Id))
                .ToListAsync();
            if (data.Count <= 0)
            {
                return false;
            }
            var deleteBy = App.User.FindFirst(ClaimConst.CLAINM_ACCOUNT)?.Value;
            var res = await db.Updateable<BPA_StoreStaff>(x => new BPA_StoreStaff() { IsDeleted = 1, DeleteAt = DateTime.Now, DeleteBy = deleteBy })
                .Where(x => ids.Contains(x.Id))
                .ExecuteCommandAsync();
            return res > 0;
        }

        [HttpPost("/api/StoreStaff/GetStoreStaffPage")]
        public async Task<PageUtil> GetStoreStaffPage(StoreStaffPageInputDto inputDto)
        {
            var total = new RefAsync<int>();
            string groupId = App.User.FindFirst(ClaimConst.GroupId)?.Value;
            var data = await db.Queryable<BPA_StoreStaff>().Where((x) => x.IsDeleted == 0&&x.GroupId==groupId)
                .WhereIF(!string.IsNullOrEmpty(inputDto.ShopId),x=>x.ShopId==inputDto.ShopId)
                .OrderBy(x => x.CreateAt, OrderByType.Desc)
                .ToPageListAsync(inputDto.Current, inputDto.PageSize, total);

            return new PageUtil()
            {
                Data = data,
                Total = total

            };
        }

        [HttpPut("/api/StoreStaff/UpdateStoreStaff")]
        public async Task<bool> UpdateStoreStaff(CreateOrUpdateStoreStaffInputDto inputDto)
        {

            var data = await db.Queryable<BPA_StoreStaff>().Where(x => x.Id == inputDto.Id&&x.IsDeleted==0).FirstAsync();

            if (data != null)
            {
                var storeStaff = await db.Queryable<BPA_StoreStaff>()
                    .Where(x => x.Number!=data.Number&& x.Number == inputDto.Number && x.IsDeleted == 0)
                    .ToListAsync();
                if (storeStaff.Count > 0)
                {
                    throw Oops.Oh($"{inputDto.Number} 职工号重复");
                }

                var updateBy = App.User.FindFirst(ClaimConst.CLAINM_ACCOUNT)?.Value;

                var res = await db.Updateable<BPA_StoreStaff>(x => new BPA_StoreStaff()
                {
                    Name = inputDto.Name,
                    Number = inputDto.Number,
                    Phone = inputDto.Phone,
                    Status = (CommonStatus)inputDto.Status,
                    ShopId= inputDto.ShopId,
                    UpdateAt = DateTime.Now,
                    UpdateBy = updateBy,
                    Pwd= data.Pwd,
                }).Where(x => x.Id == inputDto.Id).ExecuteCommandAsync();
                return res > 0;
            }

            return false;
        }

        /// <summary>
        /// 修改员工密码
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPut("/api/StoreStaff/UpdateStoreStaffPwd")]
        public async Task<bool> UpdateStoreStaffPwd(CreateOrUpdateStoreStaffInputDto inputDto)
        {
            inputDto.Pwd = MD5Encryption.Encrypt(inputDto.Pwd);
            var result = db.Updateable<BPA_StoreStaff>()
                .SetColumns(it => it.Pwd == inputDto.Pwd) //SetColumns是可以叠加的 写2个就2个字段赋值
                .Where(it => it.Id == inputDto.Id)
                .ExecuteCommand();

            return result > 0;
        }
    }
}
